﻿using System;
using System.Web.Mvc;
using Common.Log;

namespace EntornoWeb.Controllers
{
    /// <summary>
    /// Controlador de errores
    /// </summary>
    public class ErrorController : Controller
    {
        /// <summary>
        /// Gestor del error
        /// </summary>
        /// <param name="id">Código del error</param>
        /// <returns>Vista que muestra el error</returns>
        public ActionResult ErrorEndRequest(String id)
        {
            LogFactory.Escribir().Error("Redireccionando error...");

            Exception error;

            if ((this.Session != null) && (this.Session["ErrorException"] != null))
            {
                error = this.Session["ErrorException"] as Exception;
                error = error.InnerException ?? error;
                return View("ErrorDetalle", error);
            }
            else
            {
                return View("Error", new HandleErrorInfo(new Exception(HttpContext.Response.StatusDescription), "Home", "Index"));
            }
        }

        /// <summary>
        /// Capturador del error
        /// </summary>
        /// <param name="filterContext">Contexto de la excepción</param>
        protected override void OnException(ExceptionContext filterContext)
        {
            LogFactory.Escribir().Error("Inicio");
            if (filterContext == null) return;

            Exception error = filterContext.Exception ?? new Exception("Sin información adicional");
            this.Session["ErrorException"] = error;

            if (!filterContext.ExceptionHandled)
            {
                ActionResult ar = new RedirectResult("/Home/ErrorEndRequest");
                ar.ExecuteResult(filterContext);
            }

            filterContext.ExceptionHandled = true;

            LogFactory.Escribir().Error("Error: " + error.Message);
            LogFactory.Escribir().Error("Source: " + error.Source);
            LogFactory.Escribir().Error("StackTrace: " + error.StackTrace);
        }

        /// <summary>
        /// Gestor del error en peticiones ajax
        /// </summary>
        /// <param name="strMensaje">Mensaje de error que se mostrará al usuario</param>
        protected void ShowError(String strMensaje)
        {
            Response.StatusCode = 500;
            Response.StatusDescription = strMensaje;
            Response.Write(strMensaje);
            Response.ContentType = System.Net.Mime.MediaTypeNames.Text.Plain;
            Response.TrySkipIisCustomErrors = true;
        }
    }
}
