﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Common.Log;
using NHibernate_SGF.Entities;
using NHibernate_SGF.Services;

namespace EntornoWeb.Controllers
{
    /// <summary>
    /// Controlador con las funciones comunes de la aplicación
    /// </summary>
    public class SGFController : ErrorController
    {
        

        /// <summary>
        /// Capturador por defecto de todas las pantallas de la aplicación
        /// </summary>
        /// <returns>Vista por defecto</returns>
        public ActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Capturador para la generación del desplegable de colectivos
        /// </summary>
        /// <returns>Vista con los datos del desplegable</returns>
        public ActionResult ObtenerDesplegableColectivos()
        {
            LogFactory.Escribir().Info("Inicio");
            Dictionary<int, string> dicColeccion = new Dictionary<int, string>();

            List<Colectivo> lstColectivos = ColectivoService.ObtenerLista().OrderBy((Func<Colectivo, IComparable>)(a => a.Nombre)).ToList();
            foreach (Colectivo colectivo in lstColectivos)
            {
                dicColeccion.Add(colectivo.Idint, colectivo.Nombre);
            }

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstColectivos.Count + ")");
            return PartialView("Combobox", dicColeccion);
        }

        /// <summary>
        /// Capturador para la generación del desplegable de versiones
        /// </summary>
        /// <returns>Vista con los datos del desplegable</returns>
        public ActionResult ObtenerDesplegableVersiones()
        {
            LogFactory.Escribir().Info("Inicio");
            Dictionary<int, string> dicColeccion = new Dictionary<int, string>();

            List<Versio> lstVersiones = VersionService.ObtenerLista()
                .Where<Versio>(a => a.FechaBaja == DateTime.MinValue)
                .OrderBy((Func<Versio, IComparable>)(a => a.Curso.Nombre)).ToList();

            foreach (Versio version in lstVersiones)
            {
                if (version.Curso.FechaBaja == DateTime.MinValue)
                {
                    dicColeccion.Add(version.Idint, version.Curso.Nombre + " - " + version.Nombre);
                }
            }

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstVersiones.Count + ")");
            return PartialView("Combobox", dicColeccion);
        }
    }
}
