﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;

namespace EntornoWeb
{
    // Nota: para obtener instrucciones sobre cómo habilitar el modo clásico de IIS6 o IIS7, 
    // visite http://go.microsoft.com/?LinkId=9394801

    /// <summary>
    /// Proveedor del comportamiento de la aplicación web
    /// </summary>
    public class MvcApplication : System.Web.HttpApplication
    {
        /// <summary>
        /// Define la estructura de las peticiones http capturadas (rutas)
        /// </summary>
        /// <param name="routes">Colección de rutas</param>
        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default", // Nombre de ruta
                "{controller}/{action}/{id}", // URL con parámetros
                new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Valores predeterminados de parámetro
            );

        }

        /// <summary>
        /// Inicialización de la aplicación web
        /// </summary>
        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();

            RegisterRoutes(RouteTable.Routes);
        }

        /// <summary>
        /// Capturador de las peticiones http
        /// </summary>
        /// <param name="sender">Emisor de la petición</param>
        /// <param name="e">Argumentos de la petición</param>
        protected void Application_BeginRequest(object sender, EventArgs e) {
            
            try {
                string session_param_name = "ASPSESSID";
                string session_cookie_name = "ASP.NET_SessionId";
                
                if (HttpContext.Current.Request.Form[session_param_name] != null) {
                    UpdateCookie(session_cookie_name, HttpContext.Current.Request.Form[session_param_name]);
                }
                else if (HttpContext.Current.Request.QueryString[session_param_name] != null) {
                    UpdateCookie(session_cookie_name, HttpContext.Current.Request.QueryString[session_param_name]);
                }
            }
            catch { }
            
            try {
                string auth_param_name = "AUTHID";
                string auth_cookie_name = FormsAuthentication.FormsCookieName;
                
                if (HttpContext.Current.Request.Form[auth_param_name] != null) {
                    UpdateCookie(auth_cookie_name, HttpContext.Current.Request.Form[auth_param_name]);
                }
                else if (HttpContext.Current.Request.QueryString[auth_param_name] != null) {
                    UpdateCookie(auth_cookie_name, HttpContext.Current.Request.QueryString[auth_param_name]);
                }
            }
            catch { }
        }
        
        /// <summary>
        /// Alamacena la cookie indicada
        /// </summary>
        /// <param name="cookie_name">Nombre de la cookie</param>
        /// <param name="cookie_value">Valor de la cookie</param>
        private void UpdateCookie(string cookie_name, string cookie_value) {
            HttpCookie cookie = HttpContext.Current.Request.Cookies.Get(cookie_name);
            
            if (null == cookie) { 
                cookie = new HttpCookie(cookie_name);
            }
            cookie.Value = cookie_value;
            HttpContext.Current.Request.Cookies.Set(cookie);
        } 
    }
}