﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NHibernate_SGF.Entities
{
    /// <summary>
    /// Asistencia a las sesiones formativas
    /// </summary>
    public class Asistencia
    {
        private Sesion objSesion;
        private Empleado objEmpleado;
        private bool blnAsiste;

        /// <summary>
        /// Sesión vinculada a la asistencia
        /// </summary>
        public virtual Sesion Sesion
        {
            get { return objSesion; }
            set { objSesion = value; }
        }

        /// <summary>
        /// Empleado vinculado a la asistencia
        /// </summary>
        public virtual Empleado Empleado
        {
            get { return objEmpleado; }
            set { objEmpleado = value; }
        }

        /// <summary>
        /// Indica si el empleado asiste a la formación
        /// </summary>
        public virtual bool Asiste
        {
            get { return blnAsiste; }
            set { blnAsiste = value; }
        }

        /// <summary>
        /// Comprueba que la asistencia es igual a la indicada
        /// </summary>
        /// <param name="obj">Asistencia a comparar</param>
        /// <returns>Los objetos son iguales</returns>
        public override bool Equals (object obj)
        {
            if (obj == null || GetType() != obj.GetType()) 
            {
                return false;
            }

            Asistencia objAsistencia = (Asistencia)obj;
            if (this.Sesion.Idint != objAsistencia.Sesion.Idint) return false;
            if (this.Empleado.Idint != objAsistencia.Empleado.Idint) return false;
                
            return true; 
        }
    
        /// <summary>
        /// Obtiene el código hash de la asistencia
        /// </summary>
        /// <returns>Código hash</returns>
        public override int GetHashCode()
        {
            int hash = 123;
            hash += (null == this.Sesion ? 0 : this.Sesion.GetHashCode());
            hash += (null == this.Empleado ? 0 : this.Empleado.GetHashCode());
            
            return hash;
        }
        
    }
}
