﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NHibernate_SGF.Entities
{
    /// <summary>
    /// Convocatoria del empleado a la edición
    /// </summary>
    public class Convocatoria
    {
        private Edicion objEdicion;
        private Empleado objEmpleado;
        private bool blnCompletado;
        private float fltAsistencia;
        private string strObservaciones;

        /// <summary>
        /// Constructor de la convocatoria
        /// </summary>
        public Convocatoria()
        {
            this.Edicion = new Edicion();
            this.Empleado = new Empleado();
        }

        /// <summary>
        /// Edición a la que se convoca al empleado
        /// </summary>
        public virtual Edicion Edicion
        {
            get { return objEdicion; }
            set { objEdicion = value; }
        }

        /// <summary>
        /// Empleado al que se convoca a la edición
        /// </summary>
        public virtual Empleado Empleado
        {
            get { return objEmpleado; }
            set { objEmpleado = value; }
        }

        /// <summary>
        /// Indica si el empleado ha completado la edición
        /// </summary>
        public virtual bool Completado
        {
            get { return blnCompletado; }
            set { blnCompletado = value; }
        }

        /// <summary>
        /// Porcentaje de asistencia del empleado a todas las sesiones de la edición
        /// </summary>
        public virtual float Asistencia
        {
            get { return fltAsistencia; }
            set { fltAsistencia = value; }
        }

        /// <summary>
        /// Observaciones
        /// </summary>
        public virtual string Observaciones
        {
            get { return strObservaciones; }
            set { strObservaciones = value; }
        }

        /// <summary>
        /// Comprueba que la convocatoria es igual a la indicada
        /// </summary>
        /// <param name="obj">Convocatoria a comparar</param>
        /// <returns>Los objetos son iguales</returns>
        public override bool Equals (object obj)
        {
            if (obj == null || GetType() != obj.GetType()) 
            {
                return false;
            }

            Convocatoria objConvocatoria = (Convocatoria)obj;
            if (this.Edicion.Idint != objConvocatoria.Edicion.Idint) return false;
            if (this.Empleado.Idint != objConvocatoria.Empleado.Idint) return false;
                
            return true; 
        }

        /// <summary>
        /// Obtiene el código hash de la convocatoria
        /// </summary>
        /// <returns>Código hash</returns>
        public override int GetHashCode()
        {
            int hash = 123;
            hash += (null == this.Edicion ? 0 : this.Edicion.GetHashCode());
            hash += (null == this.Empleado ? 0 : this.Empleado.GetHashCode());
            
            return hash;
        }
    }
}
