﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NHibernate_SGF.Entities
{
    /// <summary>
    /// Tipos de curso
    /// </summary>
    public enum CursosTipo { Central, Local, Otros };

    /// <summary>
    /// Curso de formación a los empleados
    /// </summary>
    public class Curso
    {
        /// <summary>
        /// Literal que representa al tipo de curso Central
        /// </summary>
        protected const string STR_LITERAL_CENTRAL = "Central";
        
        /// <summary>
        /// Literal que representa al tipo de curso Local
        /// </summary>
        protected const string STR_LITERAL_LOCAL = "Local";

        /// <summary>
        /// Literal que representa al tipo de curso Otros
        /// </summary>
        protected const string STR_LITERAL_OTROS = "Otros";

        /// <summary>
        /// Abreviatura que representa al tipo de curso Central
        /// </summary>
        public const string STR_IDINT_CENTRAL = "C";
        
        /// <summary>
        /// Abreviatura que representa al tipo de curso Local
        /// </summary>
        public const string STR_IDINT_LOCAL = "L";
        
        /// <summary>
        /// Abreviatura que representa al tipo de curso Otros
        /// </summary>
        public const string STR_IDINT_OTROS = "O";

        private int intIdint;
        private string strNombre;
        private string strDescripcion;
        private string strTipo;
        private int intPeriodicidadMeses;
        private int intIncorporacionMeses;
        private bool blnRepetirConNuevaVersion;
        private DateTime dtFechaAlta;
        private DateTime dtFechaModi;
        private DateTime dtFechaBaja;
        private IList<EmpleadoCurso> lstEmpleadoCurso;
        private IList<Versio> lstVersiones;
        private IList<Planificacion> lstPlanificaciones;

        /// <summary>
        /// Identificador
        /// </summary>
        public virtual int Idint
        {
            get { return intIdint; }
            set { intIdint = value; }
        }

        /// <summary>
        /// Nombre
        /// </summary>
        public virtual string Nombre
        {
            get { return strNombre; }
            set { strNombre = value; }
        }

        /// <summary>
        /// Descripción
        /// </summary>
        public virtual string Descripcion
        {
            get { return strDescripcion; }
            set { strDescripcion = value; }
        }

        /// <summary>
        /// Tipo
        /// </summary>
        public virtual string Tipo
        {
            get { return strTipo; }
            set { strTipo = value; }
        }

        /// <summary>
        /// Periodicidad en meses con el que se debe repetir el curso
        /// </summary>
        public virtual int PeriodicidadMeses
        {
            get { return intPeriodicidadMeses; }
            set { intPeriodicidadMeses = value; }
        }

        /// <summary>
        /// Período en meses en el que se debe completar el curso tras la incorporación al trabajo
        /// </summary>
        public virtual int IncorporacionMeses
        {
            get { return intIncorporacionMeses; }
            set { intIncorporacionMeses = value; }
        }

        /// <summary>
        /// Necesidad de repetir el curso con la aparición de una nueva versión del curso
        /// </summary>
        public virtual bool RepetirConNuevaVersion
        {
            get { return blnRepetirConNuevaVersion; }
            set { blnRepetirConNuevaVersion = value; }
        }

        /// <summary>
        /// Fecha de alta
        /// </summary>
        public virtual DateTime FechaAlta
        {
            get { return dtFechaAlta; }
            set { dtFechaAlta = value; }
        }

        /// <summary>
        /// Fecha de modificación
        /// </summary>
        public virtual DateTime FechaModi
        {
            get { return dtFechaModi; }
            set { dtFechaModi = value; }
        }

        /// <summary>
        /// Fecha de baja
        /// </summary>
        public virtual DateTime FechaBaja
        {
            get { return dtFechaBaja; }
            set { dtFechaBaja = value; }
        }

        /// <summary>
        /// Lista de vinculaciones del curso con sus empleados
        /// </summary>
        public virtual IList<EmpleadoCurso> EmpleadosCursos
        {
            get { return lstEmpleadoCurso; }
            set { lstEmpleadoCurso = value; }
        }

        /// <summary>
        /// Lista de versiones del curso
        /// </summary>
        public virtual IList<Versio> Versiones
        {
            get { return lstVersiones; }
            set { lstVersiones = value; }
        }

        /// <summary>
        /// Lista de planificaciones del curso
        /// </summary>
        public virtual IList<Planificacion> Planificaciones
        {
            get { return lstPlanificaciones; }
            set { lstPlanificaciones = value; }
        }

        /// <summary>
        /// Cambia el tipo de curso
        /// </summary>
        /// <param name="enuTipo">Nuevo tipo de curso</param>
        /// <returns>Curso con tipología modificada</returns>
        public virtual Curso CambiarTipo(CursosTipo enuTipo)
        {
            Curso curso;
            switch (enuTipo)
            {
                case CursosTipo.Central:
                    curso = new CursoCentral();
                    curso.Tipo = STR_IDINT_CENTRAL;
                    break;
                case CursosTipo.Local:
                    curso = new CursoLocal();
                    curso.Tipo = STR_IDINT_LOCAL;
                    break;
                case CursosTipo.Otros:
                    curso = new CursoOtros();
                    curso.Tipo = STR_IDINT_OTROS;
                    break;
                default:
                    curso = new CursoCentral();
                    curso.Tipo = STR_IDINT_CENTRAL;
                    break;
            }
            
            curso.Idint = this.Idint;
            curso.Nombre = this.Nombre;
            curso.Descripcion = this.Descripcion;
            curso.PeriodicidadMeses = this.PeriodicidadMeses;
            curso.IncorporacionMeses = this.IncorporacionMeses;
            curso.RepetirConNuevaVersion = this.RepetirConNuevaVersion;
            curso.FechaAlta = this.FechaAlta;
            curso.FechaModi = this.FechaModi;
            curso.FechaBaja = this.FechaBaja;
            curso.EmpleadosCursos = this.EmpleadosCursos;
            return curso;
        }

        /// <summary>
        /// Obtiene la abreviatura o literal correspondiente al tipo de curso indicado
        /// </summary>
        /// <param name="strCadena">Literal o abreviatura</param>
        /// <returns>Abreviatura o literal correspondiente</returns>
        public static string CorrespondenciaTipo(string strCadena)
        {
            if (strCadena.ToUpper().Equals(STR_IDINT_CENTRAL)) return STR_LITERAL_CENTRAL;
            if (strCadena.ToUpper().Equals(STR_IDINT_LOCAL)) return STR_LITERAL_LOCAL;
            if (strCadena.ToUpper().Equals(STR_IDINT_OTROS)) return STR_LITERAL_OTROS;

            if (strCadena.ToUpper().Equals(STR_LITERAL_CENTRAL.ToUpper())) return STR_IDINT_CENTRAL;
            if (strCadena.ToUpper().Equals(STR_LITERAL_LOCAL.ToUpper())) return STR_IDINT_LOCAL;
            if (strCadena.ToUpper().Equals(STR_LITERAL_OTROS.ToUpper())) return STR_IDINT_OTROS;

            return "-";
        }

    }

    /// <summary>
    /// Curso de tipo Central
    /// </summary>
    public class CursoCentral : Curso
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public CursoCentral()
        {
            this.Tipo = STR_IDINT_CENTRAL;
        }

        /// <summary>
        /// Conversión a cadena de texto
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return STR_LITERAL_CENTRAL;
        }
    }

    /// <summary>
    /// Curso de tipo Local
    /// </summary>
    public class CursoLocal : Curso
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public CursoLocal()
        {
            this.Tipo = STR_IDINT_LOCAL;
        }

        /// <summary>
        /// Conversión a cadena de texto
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return STR_LITERAL_LOCAL;
        }
    }

    /// <summary>
    /// Curso de tipo Otros
    /// </summary>
    public class CursoOtros : Curso
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public CursoOtros()
        {
            this.Tipo = STR_IDINT_OTROS;
        }

        /// <summary>
        /// Conversión a cadena de texto
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return STR_LITERAL_OTROS;
        }
    }
}
