﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NHibernate_SGF.Entities
{
    /// <summary>
    /// Edición correspondiente a una versión
    /// </summary>
    public class Edicion
    {
        private int intIdint;
        private DateTime dtFechaInicio;
        private DateTime dtFechaFin;
        private int intSesionesDias;
        private int intDuracionMinutos;
        private string strHojaFirmasFilename;
        private byte[] binHojaFirmas;
        private Versio objVersion;
        private IList<Convocatoria> lstConvocatorias;
        private IList<Sesion> lstSesiones;

        /// <summary>
        /// Constructor
        /// </summary>
        public Edicion()
        {
            this.Version = new Versio();
            this.Convocatorias = new List<Convocatoria>();
            this.Sesiones = new List<Sesion>();
        }

        /// <summary>
        /// Identificador
        /// </summary>
        public virtual int Idint
        {
            get { return intIdint; }
            set { intIdint = value; }
        }

        /// <summary>
        /// Fecha de inicio
        /// </summary>
        public virtual DateTime FechaInicio
        {
            get { return dtFechaInicio; }
            set { dtFechaInicio = value; }
        }

        /// <summary>
        /// Fecha de fin
        /// </summary>
        public virtual DateTime FechaFin
        {
            get { return dtFechaFin; }
            set { dtFechaFin = value; }
        }

        /// <summary>
        /// Número de sesiones en días
        /// </summary>
        public virtual int SesionesDias
        {
            get { return intSesionesDias; }
            set { intSesionesDias = value; }
        }

        /// <summary>
        /// Duración de la edición en minutos
        /// </summary>
        public virtual int DuracionMinutos
        {
            get { return intDuracionMinutos; }
            set { intDuracionMinutos = value; }
        }

        /// <summary>
        /// Nombre del fichero de la hoja de firmas
        /// </summary>
        public virtual string HojaFirmasFilename
        {
            get { return strHojaFirmasFilename; }
            set { strHojaFirmasFilename = value; }
        }

        /// <summary>
        /// Fichero binario correspondiente a la hoja de firmas
        /// </summary>
        public virtual byte[] HojaFirmas
        {
            get { return binHojaFirmas; }
            set { binHojaFirmas = value; }
        }

        /// <summary>
        /// Versión asociada a la edición
        /// </summary>
        public virtual Versio Version
        {
            get { return objVersion; }
            set { objVersion = value; }
        }

        /// <summary>
        /// Lista de convocatorias a la edición
        /// </summary>
        public virtual IList<Convocatoria> Convocatorias
        {
            get { return lstConvocatorias; }
            set { lstConvocatorias = value; }
        }

        /// <summary>
        /// Lista de sesiones de la edición
        /// </summary>
        public virtual IList<Sesion> Sesiones
        {
            get { return lstSesiones; }
            set { lstSesiones = value; }
        }
    }

    /// <summary>
    /// Réplica de la clase edición sin fichero binario
    /// </summary>
    public class EdicionSinFichero
    {
        private int intIdint;
        private DateTime dtFechaInicio;
        private DateTime dtFechaFin;
        private int intSesionesDias;
        private int intDuracionMinutos;
        private string strHojaFirmasFilename;
        private Versio objVersion;
        private IList<Convocatoria> lstConvocatorias;
        private IList<Sesion> lstSesiones;

        /// <summary>
        /// Constructor
        /// </summary>
        public EdicionSinFichero()
        {
            this.Version = new Versio();
            this.Convocatorias = new List<Convocatoria>();
            this.Sesiones = new List<Sesion>();
        }

        /// <summary>
        /// Identificador
        /// </summary>
        public virtual int Idint
        {
            get { return intIdint; }
            set { intIdint = value; }
        }
        /// <summary>
        /// Fecha de inicio
        /// </summary>
        public virtual DateTime FechaInicio
        {
            get { return dtFechaInicio; }
            set { dtFechaInicio = value; }
        }

        /// <summary>
        /// Fecha de fin
        /// </summary>
        public virtual DateTime FechaFin
        {
            get { return dtFechaFin; }
            set { dtFechaFin = value; }
        }

        /// <summary>
        /// Número de sesiones en días
        /// </summary>
        public virtual int SesionesDias
        {
            get { return intSesionesDias; }
            set { intSesionesDias = value; }
        }

        /// <summary>
        /// Duración de la edición en minutos
        /// </summary>
        public virtual int DuracionMinutos
        {
            get { return intDuracionMinutos; }
            set { intDuracionMinutos = value; }
        }

        /// <summary>
        /// Nombre del fichero de la hoja de firmas
        /// </summary>
        public virtual string HojaFirmasFilename
        {
            get { return strHojaFirmasFilename; }
            set { strHojaFirmasFilename = value; }
        }

        /// <summary>
        /// Versión asociada a la edición
        /// </summary>
        public virtual Versio Version
        {
            get { return objVersion; }
            set { objVersion = value; }
        }

        /// <summary>
        /// Lista de convocatorias a la edición
        /// </summary>
        public virtual IList<Convocatoria> Convocatorias
        {
            get { return lstConvocatorias; }
            set { lstConvocatorias = value; }
        }

        /// <summary>
        /// Lista de sesiones de la edición
        /// </summary>
        public virtual IList<Sesion> Sesiones
        {
            get { return lstSesiones; }
            set { lstSesiones = value; }
        }
    }
}
