﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NHibernate_SGF.Entities
{
    /// <summary>
    /// Vinculación entre el empleado y su colectivo
    /// </summary>
    public class EmpleadoColectivo
    {
        private Empleado objEmpleado;
        private Colectivo objColectivo;
        private DateTime dtFechaAsignacion;

        /// <summary>
        /// Empleado de la vinculación
        /// </summary>
        public virtual Empleado Empleado
        {
            get { return objEmpleado; }
            set { objEmpleado = value; }
        }

        /// <summary>
        /// Colectivo de la vinculación
        /// </summary>
        public virtual Colectivo Colectivo
        {
            get { return objColectivo; }
            set { objColectivo = value; }
        }

        /// <summary>
        /// Fecha de asignación
        /// </summary>
        public virtual DateTime FechaAsignacion
        {
            get { return dtFechaAsignacion; }
            set { dtFechaAsignacion = value; }
        }

        /// <summary>
        /// Comprueba que la vinculación es igual a la indicada
        /// </summary>
        /// <param name="obj">Vinculación a comparar</param>
        /// <returns>Los objetos son iguales</returns>
        public override bool Equals (object obj)
        {
            if (obj == null || GetType() != obj.GetType()) 
            {
                return false;
            }

            EmpleadoColectivo objEmpleadoColectivo = (EmpleadoColectivo)obj;
            if (this.Colectivo.Idint != objEmpleadoColectivo.Colectivo.Idint) return false;
            if (this.Empleado.Idint != objEmpleadoColectivo.Empleado.Idint) return false;
                
            return true; 
        }

        /// <summary>
        /// Obtiene el código hash de la vinculación
        /// </summary>
        /// <returns>Código hash</returns>
        public override int GetHashCode()
        {
            int hash = 123;
            hash += (null == this.Colectivo ? 0 : this.Colectivo.GetHashCode());
            hash += (null == this.Empleado ? 0 : this.Empleado.GetHashCode());
            
            return hash;
        }
        
    }
}
