﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NHibernate_SGF.Entities
{
    /// <summary>
    /// Vinculación entre el empleado y su curso
    /// </summary>
    public class EmpleadoCurso
    {
        private Empleado objEmpleado;
        private Curso objCurso;
        private DateTime dtFechaAsignacion;
        private bool blnExento;

        /// <summary>
        /// Empleado de la vinculación
        /// </summary>
        public virtual Empleado Empleado
        {
            get { return objEmpleado; }
            set { objEmpleado = value; }
        }

        /// <summary>
        /// Curso de la vinculación
        /// </summary>
        public virtual Curso Curso
        {
            get { return objCurso; }
            set { objCurso = value; }
        }

        /// <summary>
        /// Fecha de asignación
        /// </summary>
        public virtual DateTime FechaAsignacion
        {
            get { return dtFechaAsignacion; }
            set { dtFechaAsignacion = value; }
        }

        /// <summary>
        /// Exención de realizar el curso por parte del empleado
        /// </summary>
        public virtual bool Exento
        {
            get { return blnExento; }
            set { blnExento = value; }
        }

        /// <summary>
        /// Comprueba que la vinculación es igual a la indicada
        /// </summary>
        /// <param name="obj">Vinculación a comparar</param>
        /// <returns>Los objetos son iguales</returns>
        public override bool Equals (object obj)
        {
            if (obj == null || GetType() != obj.GetType()) 
            {
                return false;
            }

            EmpleadoCurso objEmpleadoCurso = (EmpleadoCurso)obj;
            if (this.Curso.Idint != objEmpleadoCurso.Curso.Idint) return false;
            if (this.Empleado.Idint != objEmpleadoCurso.Empleado.Idint) return false;
                
            return true; 
        }

        /// <summary>
        /// Obtiene el código hash de la vinculación
        /// </summary>
        /// <returns>Código hash</returns>
        public override int GetHashCode()
        {
            int hash = 123;
            hash += (null == this.Curso ? 0 : this.Curso.GetHashCode());
            hash += (null == this.Empleado ? 0 : this.Empleado.GetHashCode());
            
            return hash;
        }
        
    }
}
