﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NHibernate_SGF.Entities
{
    /// <summary>
    /// Tipos de versión
    /// </summary>
    public enum VersionesTipo { Online, Distancia, Presencial };

    /// <summary>
    /// Versión del curso
    /// </summary>
    public class Versio
    {
        /// <summary>
        /// Literal que representa al tipo de versión Online
        /// </summary>
        protected const string STR_LITERAL_ONLINE = "Online";

        /// <summary>
        /// Literal que representa al tipo de versión A Distancia
        /// </summary>
        protected const string STR_LITERAL_DISTANCIA = "Distancia";

        /// <summary>
        /// Literal que representa al tipo de versión Presencial
        /// </summary>
        protected const string STR_LITERAL_PRESENCIA = "Presencial";
        
        /// <summary>
        /// Abreviatura que representa al tipo de versión Online
        /// </summary>
        public const string STR_IDINT_ONLINE = "O";

        /// <summary>
        /// Abreviatura que representa al tipo de versión A Distancia
        /// </summary>
        public const string STR_IDINT_DISTANCIA = "D";

        /// <summary>
        /// Abreviatura que representa al tipo de versión Presencial
        /// </summary>
        public const string STR_IDINT_PRESENCIAL = "P";

        private int intIdint;
        private string strNombre;
        private string strTipo;
        private string strTemarioFilename;
        private byte[] binTemario;
        private string strFichaFilename;
        private byte[] binFicha;
        private DateTime dtFechaAlta;
        private DateTime dtFechaModi;
        private DateTime dtFechaBaja;
        private Curso objCurso;
        private IList<Edicion> lstEdiciones;

        /// <summary>
        /// Identificador
        /// </summary>
        public virtual int Idint
        {
            get { return intIdint; }
            set { intIdint = value; }
        }

        /// <summary>
        /// Nombre
        /// </summary>
        public virtual string Nombre
        {
            get { return strNombre; }
            set { strNombre = value; }
        }

        /// <summary>
        /// Tipo de versión
        /// </summary>
        public virtual string Tipo
        {
            get { return strTipo; }
            set { strTipo = value; }
        }

        /// <summary>
        /// Nombre del fichero del temario
        /// </summary>
        public virtual string TemarioFilename
        {
            get { return strTemarioFilename; }
            set { strTemarioFilename = value; }
        }

        /// <summary>
        /// Fichero binario correspondiente al temario
        /// </summary>
        public virtual byte[] Temario
        {
            get { return binTemario; }
            set { binTemario = value; }
        }

        /// <summary>
        /// Nombre del fichero de la ficha
        /// </summary>
        public virtual string FichaFilename
        {
            get { return strFichaFilename; }
            set { strFichaFilename = value; }
        }

        /// <summary>
        /// Fichero binario correspondiente a la ficha
        /// </summary>
        public virtual byte[] Ficha
        {
            get { return binFicha; }
            set { binFicha = value; }
        }

        /// <summary>
        /// Fecha de alta
        /// </summary>
        public virtual DateTime FechaAlta
        {
            get { return dtFechaAlta; }
            set { dtFechaAlta = value; }
        }

        /// <summary>
        /// Fecha de modificación
        /// </summary>
        public virtual DateTime FechaModi
        {
            get { return dtFechaModi; }
            set { dtFechaModi = value; }
        }

        /// <summary>
        /// Fecha de baja
        /// </summary>
        public virtual DateTime FechaBaja
        {
            get { return dtFechaBaja; }
            set { dtFechaBaja = value; }
        }

        /// <summary>
        /// Curso al que pertenece la versión
        /// </summary>
        public virtual Curso Curso
        {
            get { return objCurso; }
            set { objCurso = value; }
        }

        /// <summary>
        /// Lista de ediciones de la versión
        /// </summary>
        public virtual IList<Edicion> Ediciones
        {
            get { return lstEdiciones; }
            set { lstEdiciones = value; }
        }

        /// <summary>
        /// Cambia el tipo de versión
        /// </summary>
        /// <param name="enuTipo">Nuevo tipo de versión</param>
        /// <returns>Versión con tipología modificada</returns>
        public virtual Versio CambiarTipo(VersionesTipo enuTipo)
        {
            Versio version;
            switch (enuTipo)
            {
                case VersionesTipo.Online:
                    version = new VersionOnline();
                    version.Tipo = STR_IDINT_ONLINE;
                    break;
                case VersionesTipo.Distancia:
                    version = new VersionDistancia();
                    version.Tipo = STR_IDINT_DISTANCIA;
                    break;
                case VersionesTipo.Presencial:
                    version = new VersionPresencial();
                    version.Tipo = STR_IDINT_PRESENCIAL;
                    break;
                default:
                    version = new VersionOnline();
                    version.Tipo = STR_IDINT_ONLINE;
                    break;
            }

            version.Idint = this.Idint;
            version.Nombre = this.Nombre;
            version.TemarioFilename = this.TemarioFilename;
            version.Temario = this.Temario;
            version.FichaFilename = this.FichaFilename;
            version.Ficha = this.Ficha;
            version.FechaAlta = this.FechaAlta;
            version.FechaModi = this.FechaModi;
            version.FechaBaja = this.FechaBaja;
            return version;
        }

        /// <summary>
        /// Comprueba que la versión es del tipo de versión indicada
        /// </summary>
        /// <param name="enuTipo">Tipo de versión</param>
        /// <returns>La versión es del tipo indicado</returns>
        public virtual bool EqualTipo(VersionesTipo enuTipo)
        {
            switch (enuTipo)
            {
                case VersionesTipo.Presencial:
                    if (this.Tipo.Equals(STR_IDINT_PRESENCIAL)) return true;
                    break;
                case VersionesTipo.Online:
                    if (this.Tipo.Equals(STR_IDINT_ONLINE)) return true;
                    break;
                case VersionesTipo.Distancia:
                    if (this.Tipo.Equals(STR_IDINT_DISTANCIA)) return true;
                    break;
            }
            return false;
        }

        /// <summary>
        /// Obtiene la abreviatura o literal correspondiente al tipo de versión indicada
        /// </summary>
        /// <param name="strCadena">Literal o abreviatura</param>
        /// <returns>Abreviatura o literal correspondiente</returns>
        public static string CorrespondenciaTipo(string strCadena)
        {
            if (strCadena.ToUpper().Equals(STR_IDINT_ONLINE)) return STR_LITERAL_ONLINE;
            if (strCadena.ToUpper().Equals(STR_IDINT_DISTANCIA)) return STR_LITERAL_DISTANCIA;
            if (strCadena.ToUpper().Equals(STR_IDINT_PRESENCIAL)) return STR_LITERAL_PRESENCIA;

            if (strCadena.ToUpper().Equals(STR_LITERAL_ONLINE.ToUpper())) return STR_IDINT_ONLINE;
            if (strCadena.ToUpper().Equals(STR_LITERAL_DISTANCIA.ToUpper())) return STR_IDINT_DISTANCIA;
            if (strCadena.ToUpper().Equals(STR_LITERAL_PRESENCIA.ToUpper())) return STR_IDINT_PRESENCIAL;

            return "0";
        }

    }

    /// <summary>
    /// Versión de tipo Online
    /// </summary>
    public class VersionOnline : Versio
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public VersionOnline()
        {
            this.Tipo = STR_IDINT_ONLINE;
        }

        /// <summary>
        /// Conversión a cadena de texto
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return STR_LITERAL_ONLINE;
        }
    }

    /// <summary>
    /// Versión de tipo A Distancia
    /// </summary>
    public class VersionDistancia : Versio
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public VersionDistancia()
        {
            this.Tipo = STR_IDINT_DISTANCIA;
        }

        /// <summary>
        /// Conversión a cadena de texto
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return STR_LITERAL_DISTANCIA;
        }
    }

    /// <summary>
    /// Versión de tipo Presencial
    /// </summary>
    public class VersionPresencial : Versio
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public VersionPresencial()
        {
            this.Tipo = STR_IDINT_PRESENCIAL;
        }

        /// <summary>
        /// Conversión a cadena de texto
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return STR_LITERAL_PRESENCIA;
        }
    }

}
