﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de colectivos
    /// </summary>
    public class ColectivoService : MasterService
    {
        /// <summary>
        /// Obtiene el colectivo que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intIdentificador">Identificador del colectivo</param>
        /// <returns>Colectivo</returns>
        public static Colectivo Obtener(int p_intIdentificador)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Colectivo>();
            query.Add(Restrictions.Eq("Idint", p_intIdentificador));
            Colectivo colectivo = query.UniqueResult<Colectivo>();

            LogFactory.Escribir().Info("Fin");
            return colectivo;
        }

        /// <summary>
        /// Obtiene una lista con todos los colectivos que no están dados de baja
        /// </summary>
        /// <returns>Lista de colectivos</returns>
        public static IList<Colectivo> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Colectivo>();
            query.Add(Restrictions.IsNull("FechaBaja"));
            IList<Colectivo> lstObjetos = query.List<Colectivo>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de colectivos que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="colectivo">Parámetros de filtrado de los colectivos</param>
        /// <returns>Lista de colectivos</returns>
        public static IList<Colectivo> ObtenerLista(Colectivo colectivo)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Colectivo>();
            if (colectivo.Idint != 0) query.Add(Restrictions.Eq("Idint", colectivo.Idint));
            if (colectivo.Nombre != null) query.Add(Restrictions.Like("Nombre", colectivo.Nombre, MatchMode.Anywhere));
            if (colectivo.Descripcion != null) query.Add(Restrictions.Like("Descripcion", colectivo.Descripcion, MatchMode.Anywhere));
            query.Add(Restrictions.IsNull("FechaBaja"));
            IList<Colectivo> lstObjetos = query.List<Colectivo>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Ordena una lista de colectivos en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de los colectivos</returns>
        public static Func<Colectivo, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Colectivo, IComparable> funcOrden =
                sidx == "Idint" ? a => a.Idint :
                sidx == "Nombre" ? a => a.Nombre :
                sidx == "Colectivo" ? a => a.Nombre :
                sidx == "Descripcion" ? a => a.Descripcion :
                sidx == "EsOcupacion" ? a => a.EsOcupacion :
                sidx == "FechaBaja" ? a => a.FechaBaja :
                (Func<Colectivo, IComparable>)(a => a.Nombre);
            return funcOrden;
        }
    }
}
