﻿using System;
using System.Collections.Generic;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de ediciones
    /// </summary>
    public class EdicionService : MasterService
    {
        /// <summary>
        /// Obtiene la edición que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intIdentificador">Identificador de la edición</param>
        /// <returns>Edición</returns>
        public static Edicion Obtener(int p_intIdentificador)
        { 
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Edicion>();
            query.Add(Restrictions.Eq("Idint", p_intIdentificador));
            Edicion objeto = query.UniqueResult<Edicion>();
                
            LogFactory.Escribir().Info("Fin");
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las ediciones
        /// </summary>
        /// <returns>Lista de ediciones</returns>
        public static IList<Edicion> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Edicion>();
            IList<Edicion> lstObjetos = query.List<Edicion>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista con todas las ediciones sin incluir la hoja de firmas
        /// </summary>
        /// <returns>Lista de ediciones sin incluir la hoja de firmas</returns>
        public static IList<EdicionSinFichero> ObtenerListaSinFichero()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EdicionSinFichero>();
            IList<EdicionSinFichero> lstObjetos = query.List<EdicionSinFichero>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de ediciones que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="edicion">Parámetros de filtrado de las ediciones</param>
        /// <returns>Lista de ediciones</returns>
        public static IList<Edicion> ObtenerLista(Edicion edicion)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Edicion>();
            if (edicion.Idint != 0) query.Add(Restrictions.Eq("Idint", edicion.Idint));
            if (edicion.FechaInicio.CompareTo(DateTime.MinValue) != 0) query.Add(Restrictions.Eq("FechaInicio", edicion.FechaInicio));
            if (edicion.FechaFin.CompareTo(DateTime.MinValue) != 0) query.Add(Restrictions.Eq("FechaFin", edicion.FechaFin));
            if (edicion.DuracionMinutos != 0) query.Add(Restrictions.Eq("DuracionMinutos", edicion.DuracionMinutos));
            if (edicion.SesionesDias != 0) query.Add(Restrictions.Eq("SesionesDias", edicion.SesionesDias));
            query = query.CreateCriteria("Version");
            if (edicion.Version != null && edicion.Version.Idint != 0) query.Add(Restrictions.Eq("Idint", edicion.Version.Idint));
            query = query.CreateCriteria("Curso");
            if (edicion.Version.Curso != null && edicion.Version.Curso.Nombre != null) query.Add(Restrictions.Like("Nombre", edicion.Version.Curso.Nombre, MatchMode.Anywhere));
            IList<Edicion> lstObjetos = query.List<Edicion>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de ediciones sin incluir la hoja de firmas que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="edicion">Parámetros de filtrado de las ediciones</param>
        /// <returns>Lista de ediciones sin incluir la hoja de firmas</returns>
        public static IList<EdicionSinFichero> ObtenerListaSinFichero(Edicion edicion)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EdicionSinFichero>();
            if (edicion.Idint != 0) query.Add(Restrictions.Eq("Idint", edicion.Idint));
            if (edicion.FechaInicio.CompareTo(DateTime.MinValue) != 0) query.Add(Restrictions.Eq("FechaInicio", edicion.FechaInicio));
            if (edicion.FechaFin.CompareTo(DateTime.MinValue) != 0) query.Add(Restrictions.Eq("FechaFin", edicion.FechaFin));
            if (edicion.DuracionMinutos != 0) query.Add(Restrictions.Eq("DuracionMinutos", edicion.DuracionMinutos));
            if (edicion.SesionesDias != 0) query.Add(Restrictions.Eq("SesionesDias", edicion.SesionesDias));
            query = query.CreateCriteria("Version");
            if (edicion.Version != null && edicion.Version.Idint != 0) query.Add(Restrictions.Eq("Idint", edicion.Version.Idint));
            query = query.CreateCriteria("Curso");
            if (edicion.Version.Curso != null && edicion.Version.Curso.Nombre != null) query.Add(Restrictions.Like("Nombre", edicion.Version.Curso.Nombre, MatchMode.Anywhere));
            IList<EdicionSinFichero> lstObjetos = query.List<EdicionSinFichero>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }
        
        /// <summary>
        /// Inserta la edición indicada junto con sus sesiones asociadas evitando interbloqueos por el cascade=save-update
        /// </summary>
        /// <param name="edicion">Edición a insertar</param>
        public static void Insertar(Edicion edicion)
        {
            LogFactory.Escribir().Info("Inicio");
            MasterService.Insertar(edicion);
            foreach (Sesion sesion in edicion.Sesiones)
            {
                SesionService.Insertar(sesion);
            }
            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Elimina la edición indicada junto con sus convocatorias y sesiones evitando interbloqueos por el cascade=delete-orphan
        /// </summary>
        /// <param name="edicion">Edición a eliminar</param>
        public static void Eliminar(Edicion edicion)
        {
            LogFactory.Escribir().Info("Inicio");
            foreach (Convocatoria convocatoria in edicion.Convocatorias)
            {
                ConvocatoriaService.Eliminar(convocatoria);
            }
            foreach (Sesion sesion in edicion.Sesiones)
            {
                SesionService.Eliminar(sesion);
            }
            MasterService.Eliminar(edicion);
            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Recalcula la duración, fecha de inicio y fecha de fin de la edición indicada junto con las de sus sesiones asociadas
        /// </summary>
        /// <param name="edicion">Edición a recalcular</param>
        public static void RecalcularEdicion(Edicion edicion)
        {
            LogFactory.Escribir().Info("Inicio: " + edicion.Idint);
            DateTime dtFechaInicio = DateTime.MaxValue;
            DateTime dtFechaFin = DateTime.MinValue;
            int intDuracion = 0;

            foreach (Sesion sesion in edicion.Sesiones)
            {
                if (sesion.FechaInicio < dtFechaInicio) dtFechaInicio = sesion.FechaInicio;
                if (sesion.FechaFin > dtFechaFin) dtFechaFin = sesion.FechaFin;
                intDuracion += sesion.DuracionMinutos;
            }

            edicion.FechaInicio = dtFechaInicio;
            edicion.FechaFin = dtFechaFin;
            edicion.DuracionMinutos = intDuracion;

            EdicionService.Actualizar(edicion);

            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Ordena una lista de ediciones en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las ediciones</returns>
        public static Func<EdicionSinFichero, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<EdicionSinFichero, IComparable> funcOrden =
                sidx == "Idint" ? a => a.Idint :
                sidx == "FechaInicio" ? a => a.FechaInicio :
                sidx == "FechaFin" ? a => a.FechaFin :
                sidx == "DuracionMinutos" ? a => a.DuracionMinutos :
                sidx == "SesionesDias" ? a => a.SesionesDias :
                sidx == "Version" ? a => a.Version.Nombre :
                (Func<EdicionSinFichero, IComparable>)(a => a.FechaInicio);
            return funcOrden;
        }
    }
}
