﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de vinculación entre empleados y colectivos
    /// </summary>
    public class EmpleadoColectivoService : MasterService
    {
        /// <summary>
        /// Obtiene la vinculación entre el empleado y el colectivo que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intEmpleadoIdint">Identificador del empleado</param>
        /// <param name="p_intColectivoIdint">Identificador del colectivo</param>
        /// <returns>Vinculación entre empleado y colectivo</returns>
        public static EmpleadoColectivo Obtener(int p_intEmpleadoIdint, int p_intColectivoIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EmpleadoColectivo>();
            query.CreateCriteria("Empleado", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intEmpleadoIdint));
            query.CreateCriteria("Colectivo", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intColectivoIdint));

            EmpleadoColectivo objeto = query.UniqueResult<EmpleadoColectivo>();

            LogFactory.Escribir().Info("Fin? " + (objeto != null));
            return objeto;
        }

        /// <summary>
        /// Obtiene la vinculación entre el empleado y el colectivo de tipo ocupación que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intEmpleadoIdint">Identificador del empleado</param>
        /// <returns>Vinculación entre empleado y colectivo de tipo ocupación</returns>
        public static EmpleadoColectivo ObtenerOcupacion(int p_intEmpleadoIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EmpleadoColectivo>();
            query.CreateCriteria("Empleado", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intEmpleadoIdint));
            query.CreateCriteria("Colectivo", JoinType.FullJoin).Add(Restrictions.Eq("EsOcupacion", true));

            query.SetFlushMode(FlushMode.Never);
            EmpleadoColectivo objeto = query.UniqueResult<EmpleadoColectivo>();

            LogFactory.Escribir().Info("Fin? " + (objeto != null));
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las vinculaciones entre empleados y colectivos
        /// </summary>
        /// <returns>Lista de vinculaciones entre empleados y colectivos</returns>
        public static IList<EmpleadoColectivo> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EmpleadoColectivo>();
            IList<EmpleadoColectivo> lstObjetos = query.List<EmpleadoColectivo>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista con todas las vinculaciones entre empleados y colectivos que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="empleadoColectivo">Parámetros de filtrado de las vinculaciones</param>
        /// <returns>Lista de vinculaciones entre empleados y colectivos</returns>
        public static IList<EmpleadoColectivo> ObtenerLista(EmpleadoColectivo empleadoColectivo)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EmpleadoColectivo>();
            if (empleadoColectivo.Empleado != null && empleadoColectivo.Empleado.Idint != 0) query.Add(Restrictions.Eq("Empleado.Idint", empleadoColectivo.Empleado.Idint));
            if (empleadoColectivo.Colectivo != null && empleadoColectivo.Colectivo.Idint != 0) query.Add(Restrictions.Eq("Colectivo.Idint", empleadoColectivo.Colectivo.Idint));
            query = query.CreateCriteria("Colectivo", JoinType.LeftOuterJoin);
            if (empleadoColectivo.Colectivo != null && empleadoColectivo.Colectivo.Nombre != null) query.Add(Restrictions.Like("Nombre", empleadoColectivo.Colectivo.Nombre, MatchMode.Anywhere));

            IList<EmpleadoColectivo> lstObjetos = query.List<EmpleadoColectivo>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Ordena una lista de vinculaciones entre empleado y colectivo en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las vinculaciones entre empleado y colectivo</returns>
        public static Func<EmpleadoColectivo, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<EmpleadoColectivo, IComparable> funcOrden =
                sidx == "Colectivo" ? a => a.Colectivo.Nombre :
                sidx == "Empleado" ? a => a.Empleado.Nombre :
                (Func<EmpleadoColectivo, IComparable>)(a => a.Colectivo.Nombre);
            return funcOrden;
        }
    }
}
