﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de vinculación entre empleados y cursos
    /// </summary>
    public class EmpleadoCursoService : MasterService
    {
        /// <summary>
        /// Obtiene la vinculación entre el empleado y el curso que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intEmpleadoIdint">Identificador del empleado</param>
        /// <param name="p_intCursoIdint">Identificador del curso</param>
        /// <returns>Vinculación entre empleado y curso</returns>
        public static EmpleadoCurso Obtener(int p_intEmpleadoIdint, int p_intCursoIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EmpleadoCurso>();
            query.CreateCriteria("Empleado", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intEmpleadoIdint));
            query.CreateCriteria("Curso", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intCursoIdint));

            EmpleadoCurso objeto = query.UniqueResult<EmpleadoCurso>();

            LogFactory.Escribir().Info("Fin? " + (objeto != null));
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las vinculaciones entre empleados y cursos
        /// </summary>
        /// <returns>Lista de vinculaciones entre empleados y cursos</returns>
        public static IList<EmpleadoCurso> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EmpleadoCurso>();
            IList<EmpleadoCurso> lstObjetos = query.List<EmpleadoCurso>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista con todas las vinculaciones entre empleados y cursos que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="empleadoCurso">Parámetros de filtrado de las vinculaciones</param>
        /// <returns>Lista de vinculaciones entre empleados y cursos</returns>
        public static IList<EmpleadoCurso> ObtenerLista(EmpleadoCurso empleadoCurso)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<EmpleadoCurso>();
            if (empleadoCurso.Empleado != null && empleadoCurso.Empleado.Idint != 0) query.Add(Restrictions.Eq("Empleado.Idint", empleadoCurso.Empleado.Idint));
            if (empleadoCurso.Curso != null && empleadoCurso.Curso.Idint != 0) query.Add(Restrictions.Eq("Curso.Idint", empleadoCurso.Curso.Idint));
            query = query.CreateCriteria("Curso", JoinType.LeftOuterJoin);
            if (empleadoCurso.Curso != null && empleadoCurso.Curso.Nombre != null) query.Add(Restrictions.Like("Nombre", empleadoCurso.Curso.Nombre, MatchMode.Anywhere));
            if (empleadoCurso.Curso != null && empleadoCurso.Curso.Tipo != null) query.Add(Restrictions.Eq("Tipo", Curso.CorrespondenciaTipo(empleadoCurso.Curso.Tipo)));
            query.Add(Restrictions.IsNull("FechaBaja"));

            IList<EmpleadoCurso> lstObjetos = query.List<EmpleadoCurso>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Ordena una lista de vinculaciones entre empleado y curso en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las vinculaciones entre empleado y curso</returns>
        public static Func<EmpleadoCurso, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<EmpleadoCurso, IComparable> funcOrden =
                sidx == "Curso" ? a => a.Curso.Nombre :
                sidx == "Empleado" ? a => a.Empleado.Nombre :
                sidx == "FechaAsignacion" ? a => a.FechaAsignacion :
                sidx == "Exento" ? a => a.Exento :
                (Func<EmpleadoCurso, IComparable>)(a => a.Curso.Nombre);
            return funcOrden;
        }
    }
}
