﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de planificaciones
    /// </summary>
    public class PlanificacionService : MasterService
    {
        /// <summary>
        /// Obtiene la planificación que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intIdentificador">Identificador de la planificación</param>
        /// <returns>Planificación</returns>
        public static Planificacion Obtener(int p_intIdentificador)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Planificacion>();
            query.Add(Restrictions.Eq("Idint", p_intIdentificador));
            Planificacion objeto = query.UniqueResult<Planificacion>();

            LogFactory.Escribir().Info("Fin");
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las planificaciones
        /// </summary>
        /// <returns>Lista de planificaciones</returns>
        public static IList<Planificacion> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Planificacion>();
            query.Add(Restrictions.IsNull("FechaBaja"));
            IList<Planificacion> lstObjetos = query.List<Planificacion>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de planificaciones que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="planificacion">Parámetros de filtrado de las planificaciones</param>
        /// <returns>Lista de planificaciones</returns>
        public static IList<Planificacion> ObtenerLista(Planificacion planificacion)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Planificacion>();
            if (planificacion.Idint != 0) query.Add(Restrictions.Eq("Idint", planificacion.Idint));
            if (planificacion.Nombre != null) query.Add(Restrictions.Like("Nombre", planificacion.Nombre, MatchMode.Anywhere));
            query.Add(Restrictions.IsNull("FechaBaja"));
            query = query.CreateCriteria("Curso");
            if (planificacion.Curso != null && planificacion.Curso.Idint != 0) query.Add(Restrictions.Eq("Idint", planificacion.Curso.Idint));
            IList<Planificacion> lstObjetos = query.List<Planificacion>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Ordena una lista de planificaciones en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las planificaciones</returns>
        public static Func<Planificacion, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Planificacion, IComparable> funcOrden =
                sidx == "Idint" ? a => a.Idint :
                sidx == "Nombre" ? a => a.Nombre :
                sidx == "Descripcion" ? a => a.Descripcion :
                sidx == "FechaAlta" ? a => a.FechaAlta :
                sidx == "FechaBaja" ? a => a.FechaBaja :
                (Func<Planificacion, IComparable>)(a => a.Nombre);
            return funcOrden;
        }
    }
}
