﻿using System;
using System.Collections.Generic;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de sesiones
    /// </summary>
    public class SesionService : MasterService
    {
        /// <summary>
        /// Obtiene la sesión que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intIdentificador">Identificador de la sesión</param>
        /// <returns>Sesión</returns>
        public static Sesion Obtener(int p_intIdentificador)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Sesion>();
            query.Add(Restrictions.Eq("Idint", p_intIdentificador));
            Sesion objeto = query.UniqueResult<Sesion>();

            LogFactory.Escribir().Info("Fin");
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las sesiones
        /// </summary>
        /// <returns>Lista de sesiones</returns>
        public static IList<Sesion> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Sesion>();
            IList<Sesion> lstObjetos = query.List<Sesion>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de sesiones que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="sesion">Parámetros de filtrado de las sesiones</param>
        /// <returns>Lista de sesiones</returns>
        public static IList<Sesion> ObtenerLista(Sesion sesion)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Sesion>();
            if (sesion.Idint != 0) query.Add(Restrictions.Eq("Idint", sesion.Idint));
            query = query.CreateCriteria("Edicion");
            if (sesion.Edicion != null & sesion.Edicion.Idint != 0) query.Add(Restrictions.Eq("Idint", sesion.Edicion.Idint));
            IList<Sesion> lstObjetos = query.List<Sesion>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        //Eliminacion en cascada pq no podemos usar el cascade=delete-orphan por interbloqueos
        /// <summary>
        /// Elimina la sesión que coincide con los parámetros indicados junto con sus asistencias de empleados
        /// </summary>
        /// <param name="sesion">Sesión a eliminar</param>
        public static void Eliminar(Sesion sesion)
        {
            LogFactory.Escribir().Info("Inicio");
            foreach (Asistencia asistencia in sesion.Asistencias)
            {
                AsistenciaService.Eliminar(asistencia);
            }
            MasterService.Eliminar(sesion);
            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Ordena una lista de sesiones en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las sesiones</returns>
        public static Func<Sesion, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Sesion, IComparable> funcOrden =
                sidx == "Idint" ? a => a.Idint :
                sidx == "Numero" ? a => a.Numero :
                sidx == "FechaInicio" ? a => a.FechaInicio :
                sidx == "FechaFin" ? a => a.FechaFin :
                sidx == "Ubicacion" ? a => a.Ubicacion :
                sidx == "DuracionMinutos" ? a => a.DuracionMinutos :
                sidx == "FechaInicio" ? a => a.Edicion.FechaInicio :
                sidx == "FechaFin" ? a => a.Edicion.FechaFin :
                (Func<Sesion, IComparable>)(a => a.FechaInicio);
            return funcOrden;
        }
    }
}
